package de.fraunhofer.iem.dataprovider.git

import jakarta.annotation.PostConstruct
import jakarta.annotation.PreDestroy
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import org.springframework.stereotype.Component
import kotlin.coroutines.CoroutineContext

@Component
class TaskManager : CoroutineScope {

    private val job = SupervisorJob()
    override val coroutineContext: CoroutineContext
        get() = Dispatchers.Default + job

    // TODO: this could be dangerous to give it unlimited memory
    private val tasks = Channel<String>(Channel.UNLIMITED)

    @PostConstruct
    fun startWorkers() {
        // TODO: spawn threads equal to number of cores
        repeat(10) {
            println("[${Thread.currentThread().name}] launching coroutine")
            launchWorker(it)
        }
    }

    private fun launchWorker(id: Int) = launch {
        for (task in tasks) {
            println("[${Thread.currentThread().name}] Processor #$id received $task")
            executeTask(task)
            println("[${Thread.currentThread().name}] Processor #$id finished $task")
        }
    }

    private suspend fun executeTask(task: String) {
        // simulate a long running task
        delay(5000)
    }

    fun addTask(task: String) {
        launch {
            tasks.send(task)
        }
    }

    @PreDestroy
    fun stop() {
        job.cancel()
    }

}
